<?php
/*
Plugin Name: Repco RSS Feed
Plugin URI: https://cba.media
Description: Make your RSS Feeds compatible with Repco. Add pagination, featured image, the whole text and additional post types to your RSS Feeds.
Version: 0.91
Author: Ingo Leindecker
Author URI: https://cba.media
*/


/**
 * Allow limiting and pagination in any regular WP feed
 */
function rbwo4nvdlfopf_allow_feed_limit( $limits ) {

  if ( ! is_feed() )
    return $limits;

  $limit = isset( $_GET['limit'] ) && is_numeric( $_GET['limit'] ) ? $_GET['limit'] : 100;
  $offset = isset( $_GET['offset'] ) && is_numeric( $_GET['offset'] ) ? $_GET['offset'] : 0;

  if( $limit < 1 || $limit > 100 )
    $limit = 100;

  return "LIMIT " . esc_sql( $offset ) . "," . esc_sql( $limit );

}
add_filter( 'post_limits', 'rbwo4nvdlfopf_allow_feed_limit' );


/*
 * Add the post thumbnail to the <item>'s image tag
 */
function rbwo4nvdlfopf_add_thumbnail_to_rss_item() {

	global $post;

	if( ! has_post_thumbnail( $post->ID ) )
		return;

	$thumbnail_id = get_post_thumbnail_id( $post->ID );
	$thumbnail = get_post( $thumbnail_id );

	echo '<image>';
		echo '<title>' . apply_filters( 'the_title_rss', $thumbnail->post_title ) . '</title>'; // Thumbnail's title
		echo '<description>' . apply_filters( 'the_content_feed', $thumbnail->post_content ) . '</description>'; // Thumbnail's description
		echo '<url>' . get_the_post_thumbnail_url( $post->ID ) . '</url>'; // Thumbnail URL
		echo '<link>' . get_permalink( $post->ID ) . '</link>'; // Permalink to the parent post
	echo '</image>';

}
add_action( 'rss2_item', 'rbwo4nvdlfopf_add_thumbnail_to_rss_item' );


/**
 * Add the post types defined in the settings to the main RSS feed
 */
function rbwo4nvdlfopf_add_feeds_for_custom_post_types( $request ) {

	// Add all post types that were defined in the settings to the feed
	if( isset( $request['feed']) && ! isset( $request['post_type'] ) ) {
		$request['post_type'] = get_option( 'repco_rss_post_types', array( 'post' ) );
	}

	return $request;
}
add_filter( 'request', 'rbwo4nvdlfopf_add_feeds_for_custom_post_types' );


/**
 * Plugin activation
 *
 * Sets 'rss_use_excerpt' to 1: This shows the whole text in the <item>'s <description>-tag
 * Adds additional post types to the feed as defined in the settings
 */
function rbwo4nvdlfopf_plugin_activation() {
	// Make sure to display the whole text in the RSS feed
	update_option( 'rss_use_excerpt', '0' );

	$repco_rss_post_types = get_option( 'repco_rss_post_types' );
	if( ! $repco_rss_post_types || empty( $repco_rss_post_types ) )
		update_option( 'repco_rss_post_types', array( 'post') );

}
register_activation_hook( __FILE__,  'rbwo4nvdlfopf_plugin_activation' );



/**
 * Register and add settings
 */
add_action( 'admin_menu', 'rbwo4nvdlfopf_add_plugin_page' );
add_action( 'admin_init', 'rbwo4nvdlfopf_page_init' );

function rbwo4nvdlfopf_page_init() {

   register_setting(
      'rbwo4nvdlfopf_option_group', // Option group
      'repco_rss_post_types', // Option name
      'rbwo4nvdlfopf_sanitize' // Sanitize
   );

   add_settings_section(
      'settings_repco_rss', // ID
      __( 'Include Post Types in RSS Feed' ), // Title
      'rbwo4nvdlfopf_print_section_info', // Callback
      'repco-rss-feeds' // Page
   );

   add_settings_field(
      'repco_rss_post_types', // ID
      __( 'Include Post Types in RSS Feed' ), // Title
      'rbwo4nvdlfopf_post_types_callback', // Callback
      'repco-rss-feeds', // Page
      'settings_repco_rss' // Section
   );

}


/**
 * Add options page
 */
function rbwo4nvdlfopf_add_plugin_page() {
   // This page will be under "Settings"
   add_options_page(
       'Settings Admin',
       __( 'Repco RSS Feeds'),
       'manage_options',
       'repco-rss-feeds',
       'rbwo4nvdlfopf_create_admin_page'
   );
}



/**
 * Options page callback
 */
function rbwo4nvdlfopf_create_admin_page() {
   // Set class property

   ?>
   <div class="wrap">
       <h1>Repco RSS Feeds</h1>
       <form method="post" action="options.php">
       <?php
           // This prints out all hidden setting fields
           settings_fields( 'rbwo4nvdlfopf_option_group' );

           do_settings_sections( 'repco-rss-feeds' );
       ?>
       </div>
       <?php
           submit_button();
       ?>
       </form>
   </div>
<?php
}

/**
 * Sanitize each setting field as needed
 *
 * @param array $input Contains all settings fields as array keys
 */
function rbwo4nvdlfopf_sanitize( $input ) {
	if( ! is_array( $input ) )
		return array();

	// Remove empty values and duplicates
	return array_unique( array_filter( $input, function( $value ) { return ! is_null( $value ) && ! empty( $value ); } ) );
}

/**
 * Print the Section text
 */
function rbwo4nvdlfopf_print_section_info() {
   // print 'Enter your settings below:';
}

/**
 * Get the settings option array and print one of its values
 */
function rbwo4nvdlfopf_post_types_callback() {
	/**
	* Default post types
	*/

	$value = get_option( 'repco_rss_post_types', array( 'post' ) );
	$post_types = get_post_types();

	// Exclude certain post types generally that don't come into consideration
	$exclude = array(
		'attachment',
		'revision',
		'nav_menu_item',
		'custom_css',
		'customize_changeset',
		'oembed_cache',
		'wp_block',
		'wp_template',
		'wp_template_part',
		'wp_global_styles',
		'wp_navigation',
		'acf-taxonomy',
		'acf-post-type',
		'acf-field-group',
		'acf-field',
		'taxopress_logs',
		'user_request',
		'elementor_library',
		'feedzy_imports',
		'feedzy_categories',
		'ap_follower',
		'elementor_font',
		'elementor_icons',
		'ppma_boxes',
		'ppmacf_field'
	);

	$post_types = array_diff( $post_types, $exclude );

	$i = 0;

	foreach( $value as $key => $val ) {

		?>
		<select id="repco-rss-post-types-<?php echo $i; ?>" name="repco_rss_post_types[]">
		<?php if( $i > 0 ): ?><option value="">-<?php _e( 'None' ); ?>-</option><?php endif; ?>
		<?php foreach( $post_types as $post_type ) :
			$selected = $val == $post_type ? 'selected="selected"' : ''; ?>
			<option value="<?php echo $post_type; ?>" <?php echo $selected; ?>><?php echo $post_type; ?></option>
		<?php endforeach; ?>
		</select>
		<?php

		$i++;

	}

	?>

	<select id="repco-rss-post-types-<?php echo $i; ?>" name="repco_rss_post_types[]">
		<option value="">-<?php _e( 'Add new', 'trp' ); ?>-</option>
		<?php foreach( $post_types as $post_type ) : ?>
			<option value="<?php echo $post_type; ?>"><?php echo $post_type; ?></option>
		<?php endforeach; ?>
	</select>

	<p class="description"><?php esc_html_e( 'Choose which post types to include your main RSS feed.', 'trp' ); ?></p>
	<?php

}

?>